<?php

namespace Drupal\config_actions_provider\Plugin\ConfigProvider;

use Drupal\config_actions\ConfigActionsSourceManager;
use Drupal\config_actions\ConfigActionsServiceInterface;
use Drupal\config_actions_provider\Plugin\ConfigActionsSource\ConfigActionsProvider;
use Drupal\config_provider\InMemoryStorage;
use Drupal\config_provider\Plugin\ConfigProviderBase;
use Drupal\Core\Config\StorageInterface;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Class for providing configuration actions.
 *
 * @ConfigProvider(
 *   id = \Drupal\config_actions_provider\Plugin\ConfigProvider\ConfigProviderActions::ID,
 *   weight = 100,
 *   label = @Translation("Actions"),
 *   description = @Translation("Actions to be applied to configuration provided by other modules and themes."),
 * )
 */
class ConfigProviderActions extends ConfigProviderBase implements ContainerFactoryPluginInterface {

  /**
   * The configuration provider ID.
   */
  const ID = 'config/actions';

  /**
   * The module handler.
   *
   * @var \Drupal\Core\Extension\ModuleHandlerInterface
   */
  protected $moduleHandler;

  /**
   * The config actions service.
   *
   * @var \Drupal\config_actions\ConfigActionsServiceInterface
   */
  protected $configActionsService;

  /**
   * The config actions source manager.
   *
   * @var \Drupal\config_actions\ConfigActionsSourceManager
   */
  protected $configActionsSourceManager;

  /**
   * Constructs a new ConfitProviderActions.
   *
   * @param array $configuration
   *   A configuration array containing information about the plugin instance.
   * @param string $plugin_id
   *   The plugin_id for the plugin instance.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   * @param \Drupal\Core\Extension\ModuleHandlerInterface $module_handler
   *   The module handler.
   * @param ConfigActionsServiceInterface $config_actions_service
   *   The ConfigActionsService from the container.
   * @param \Drupal\config_actions\ConfigActionsSourceManager $config_actions_source_manager
   *   The config actions source manager.
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition, ModuleHandlerInterface $module_handler, ConfigActionsServiceInterface $config_actions_service, ConfigActionsSourceManager $config_actions_source_manager) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->moduleHandler = $module_handler;
    $this->configActionsService = $config_actions_service;
    $this->configActionsSourceManager = $config_actions_source_manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('module_handler'),
      $container->get('config_actions'),
      $container->get('plugin.manager.config_actions_source')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function addConfigToCreate(array &$config_to_create, StorageInterface $storage, $collection, $prefix = '', array $profile_storages = []) {
    // No action needed as config_actions already acts on extension install.
  }

  /**
   * {@inheritdoc}
   */
  public function addInstallableConfig(InMemoryStorage $installable_config, array $extensions = []) {
    $module_list = $this->listModulesInDependencyOrder();
    // Run actions in module dependency order. It is safe to call
    // ::importAction() on a module that doesn't provide actions.
    foreach ($module_list as $module_name) {
      // We don't need to pass the $install_storage, as it's already
      // injected into the ConfigActionsProvider.
      $this->configActionsService->importAction($module_name, '', '', ['source_type' => ConfigActionsProvider::ID]);
    }
  }

  /**
   * Returns a list of all installed modules sorted in order of dependency.
   *
   * @return string[]
   *   An array of module names.
   */
  protected function listModulesInDependencyOrder() {
    // Get all installed modules.
    $module_list = array_keys($this->moduleHandler->getModuleList());
    $module_list = array_combine($module_list, $module_list);

    // Get a list of modules with dependency weights as values.
    $module_data = system_rebuild_module_data();
    // Set the actual module weights.
    $module_list = array_map(function ($module) use ($module_data) {
      return $module_data[$module]->sort;
    }, $module_list);

    // Sort the module list by their weights (reverse).
    arsort($module_list);
    return array_keys($module_list);
  }
}
